@@ -33,23 +33,25 @@ def _simplify(stream, with_attrs=False):
         for mark, (kind, data, pos) in stream:
             if kind is START:
                 if with_attrs:
-                    data = (unicode(data[0]), dict([(unicode(k), v)
-                                                    for k, v in data[1]]))
+                    data = (unicode(data[0]), dict((unicode(k), v)
+                                                   for k, v in data[1]))
                 else:
                     data = unicode(data[0])
             elif kind is END:
                 data = unicode(data)
             elif kind is ATTR:
                 kind = ATTR
-                data = dict([(unicode(k), v) for k, v in data[1]])
+                data = dict((unicode(k), v) for k, v in data[1])
             yield mark, kind, data
     return list(_generate())
 
 
 def _transform(html, transformer, with_attrs=False):
     """Apply transformation returning simplified marked stream."""
-    if isinstance(html, basestring):
-        html = HTML(html)
+    if isinstance(html, basestring) and not isinstance(html, unicode):
+        html = HTML(html, encoding='utf-8')
+    elif isinstance(html, unicode):
+        html = HTML(html, encoding='utf-8')
     stream = transformer(html, keep_marks=True)
     return _simplify(stream, with_attrs)
 
@@ -57,7 +59,7 @@ def _transform(html, transformer, with_attrs=False):
 class SelectTest(unittest.TestCase):
     """Test .select()"""
     def _select(self, select):
-        html = HTML(FOOBAR)
+        html = HTML(FOOBAR, encoding='utf-8')
         if isinstance(select, basestring):
             select = [select]
         transformer = Transformer(select[0])
@@ -138,7 +140,7 @@ class SelectTest(unittest.TestCase):
 
     def test_select_text_context(self):
         self.assertEqual(
-            list(Transformer('.')(HTML('foo'), keep_marks=True)),
+            list(Transformer('.')(HTML(u'foo'), keep_marks=True)),
             [('OUTSIDE', ('TEXT', u'foo', (None, 1, 0)))],
             )
 
@@ -205,7 +207,7 @@ class InvertTest(unittest.TestCase):
 
     def test_invert_text_context(self):
         self.assertEqual(
-            _simplify(Transformer('.').invert()(HTML('foo'), keep_marks=True)),
+            _simplify(Transformer('.').invert()(HTML(u'foo'), keep_marks=True)),
             [(None, 'TEXT', u'foo')],
             )
 
@@ -271,7 +273,7 @@ class EmptyTest(unittest.TestCase):
 
     def test_empty_text_context(self):
         self.assertEqual(
-            _simplify(Transformer('.')(HTML('foo'), keep_marks=True)),
+            _simplify(Transformer('.')(HTML(u'foo'), keep_marks=True)),
             [(OUTSIDE, TEXT, u'foo')],
             )
 
@@ -522,6 +524,15 @@ class FilterTest(unittest.TestCase):
             self._filter('.', 'foo'),
             [[(None, TEXT, u'foo')]])
 
+    def test_filter_after_outside(self):
+        stream = _transform(
+            '<root>x</root>', Transformer('//root/text()').filter(lambda x: x))
+        self.assertEqual(
+            list(stream),
+            [(None, START, u'root'),
+             (OUTSIDE, TEXT, u'x'),
+             (None, END, u'root')])
+
 
 class MapTest(unittest.TestCase):
     def _map(self, select, kind=None):
@@ -535,29 +546,29 @@ class MapTest(unittest.TestCase):
     def test_map_element(self):
         self.assertEqual(
             self._map('foo'),
-            [(QName(u'foo'), Attrs([(QName(u'name'), u'foo'),
-                                    (QName(u'size'), u'100')])),
+            [(QName('foo'), Attrs([(QName('name'), u'foo'),
+                                   (QName('size'), u'100')])),
              u'FOO',
-             QName(u'foo')]
-            )
+             QName('foo')]
+        )
 
     def test_map_with_text_kind(self):
         self.assertEqual(
             self._map('.', TEXT),
             [u'ROOT', u'FOO', u'BAR']
-            )
+        )
 
     def test_map_with_root_and_end_kind(self):
         self.assertEqual(
             self._map('.', END),
-            [QName(u'foo'), QName(u'bar'), QName(u'root')]
-            )
+            [QName('foo'), QName('bar'), QName('root')]
+        )
 
     def test_map_with_attribute(self):
         self.assertEqual(
             self._map('foo/@name'),
-            [(QName(u'foo@*'), Attrs([('name', u'foo')]))]
-            )
+            [(QName('foo@*'), Attrs([('name', u'foo')]))]
+        )
 
 
 class SubstituteTest(unittest.TestCase):
@@ -647,9 +658,11 @@ class ContentTestMixin(object):
 
             def __iter__(self):
                 self.count += 1
-                return iter(HTML('CONTENT %i' % self.count))
+                return iter(HTML(u'CONTENT %i' % self.count))
 
-        if isinstance(html, basestring):
+        if isinstance(html, basestring) and not isinstance(html, unicode):
+            html = HTML(html, encoding='utf-8')
+        else:
             html = HTML(html)
         if content is None:
             content = Injector()
